package com.sundata.internalevaluation.script.execute;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import groovy.text.Template;
import org.apache.commons.pool2.ObjectPool;
import com.sundata.internalevaluation.script.cache.ScriptLRUCache;
import com.sundata.internalevaluation.script.model.TemplateModel;
import com.sundata.internalevaluation.script.pool.GroovyTemplatePool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

public class TemplateExecute {
    private static final Logger log = LoggerFactory.getLogger(TemplateExecute.class);
    private static final ScriptLRUCache<TemplateModel, ObjectPool<Template>> cache = new ScriptLRUCache<>(200);

    private static void addCache(TemplateModel model, ObjectPool<Template> pool) {
        cache.put(model, pool);
    }

    public static void addCache(String templateId, String tempalteStr) {
        TemplateModel model = new TemplateModel(templateId, tempalteStr);
        ObjectPool<Template> pool = GroovyTemplatePool.templateFactoryObjectPool(model);
        addCache(model, pool);
    }

    /**
     * 根据数据对象对缓存进行处理，使用一次就处理一次
     *
     * @param model 脚本对象
     * @return 池对象
     */
    public static ObjectPool<Template> getPool(TemplateModel model) {
        if (cache.containsKey(model)) {
            return cache.get(model);
        } else {
            ObjectPool<Template> pool = GroovyTemplatePool.templateFactoryObjectPool(model);
            addCache(model, pool);
            return cache.get(model);
        }
    }

    /**
     * 根据ID获取缓存中的数据内容，不建议直接使用，需要确保数据中存在才行
     *
     * @param templateId 脚本ID
     * @return 脚本池
     */
    public static ObjectPool<Template> getPoolById(String templateId) {
        AtomicReference<ObjectPool<Template>> pool = new AtomicReference<>();
        cache.forEach((c, p) -> {
            if (templateId.equals(c.getTemplateId())) {
                pool.set(p);
            }
        });
        if (pool.get() != null) {
            return pool.get();
        } else {
            // 缓存级数据没有就没有，没有需要重新构建
            return null;
        }
    }

    /**
     * 根据ID获取缓存中的数据内容，不建议直接使用，需要确保数据中存在才行
     *
     * @param templateStr 脚本内容
     * @return 脚本池
     */
    public static ObjectPool<Template> getPoolByScriptStr(String templateStr) {
        AtomicReference<ObjectPool<Template>> pool = new AtomicReference<>();
        cache.forEach((c, p) -> {
            if (templateStr.equals(c.getTemplateStr())) {
                pool.set(p);
            }
        });
        if (pool.get() != null) {
            return pool.get();
        } else {
            // 缓存级数据没有就没有，没有需要重新构建
            return null;
        }
    }

    public static boolean hasCacheById(String scriptId) {
        return getPoolById(scriptId) != null;
    }

    public static boolean hasCacheByScriptStr(String scriptStr) {
        return getPoolByScriptStr(scriptStr) != null;
    }

    public static boolean hasCache(String scriptId, String scriptStr) {
        return getPool(new TemplateModel(scriptId, scriptStr)) != null;
    }

    /**
     * 计算主方法
     *
     * @param templateId  计算id
     * @param templateStr 脚本类
     * @param params      脚本参数
     * @return 脚本计算结果
     */
    public static String executeScript(String templateId, String templateStr, Map<String, Object> params) {
        AtomicReference<TemplateModel> model = new AtomicReference<>();
        if (StrUtil.isNotBlank(templateId) && hasCacheById(templateId)) {
            String finalTemplateId = templateId;
            cache.keySet().forEach((k) -> {
                if (k.getTemplateId().equals(finalTemplateId)) {
                    model.set(k);
                    if (k.getTemplateStr().equals(templateStr)) {
                        model.set(k);
                    }else{
                        cache.remove(model);
                        model.set(new TemplateModel(finalTemplateId,templateStr));
                    }
                }
            });
            return executeScript(model.get(), params);
        } else if (StrUtil.isNotBlank(templateStr) && hasCacheByScriptStr(templateStr)) {
            cache.keySet().forEach((k) -> {
                if (k.getTemplateStr().equals(templateStr)) {
                    model.set(k);
                }
            });
            return executeScript(model.get(), params);
        } else if (hasCache(templateId, templateStr) && StrUtil.isNotBlank(templateStr)) {
            return executeScript(new TemplateModel(templateId, templateStr), params);
        } else {
            if (StrUtil.isBlank(templateStr) && StrUtil.isBlank(templateId)) {
                throw new NullPointerException("脚本与id均为空，无法执行计算");
            }
            if (StrUtil.isBlank(templateId)) {
                templateId = RandomUtil.randomString(10);
            }
            return executeScript(new TemplateModel(templateId, templateStr), params);
        }


    }

    /**
     * 计算的基础方法
     *
     * @param scriptModel 对象清单
     * @param params      计算用脚本参数
     * @return 计算结果
     */
    private static String executeScript(TemplateModel scriptModel, Map<String, Object> params) {
        if (scriptModel == null) {
            throw new NullPointerException("scriptModel is null");
        }
        ObjectPool<Template> pool = getPool(scriptModel);
        try {
            Template template = pool.borrowObject();

            return template.make(params).toString();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        throw new RuntimeException("计算失败，请检查实际执行情况与日志！");
    }
}
